import { currentAccount, connect, nativeBalance, getChain } from "../utility/wallet.js";
import { asUnit, fromWei } from "../utility/ethers.js";
import { initializeContracts, v2routerContract } from "../contract/manager.js";
import { ERC20 } from "../contract/ERC20.js";
import { BaseElement } from "@ohlabs/js-core/utility/element/base-element.js";
import { swapTemplate } from "@ohlabs/js-core/utility/document/template/swap-template.js";
import { tokenConfig } from "@ohlabs/configuration/token.js";

class Swap extends BaseElement {  

    slippage;

    tokenA;
    tokenAName;
    tokenSwapA;
    balanceA;
    cachebalanceA;

    swapDirection;

    tokenB;
    tokenBName;
    tokenSwapB;
    balanceB;
    cachebalanceB;

    quoteWait;

    swapBtn;
    swapError;

    constructor(){ 
        super();
    }

    connectedCallback() {
        super.shadowRoot = this.attachShadow({ mode: "open" });
        this.loadTemplate();
    }

    loadTemplate(){
        if(!this.loaded){
                super.shadowRoot.innerHTML = swapTemplate;
                super.loadCss();

                this.slippage = this.root.getElementById('slippage');
                this.slippage.value = 1;

                this.tokenAName = this.root.getElementById('aToken');
                this.tokenSwapA = this.root.getElementById('tokenSwapA');
                this.tokenSwapA.addEventListener('input', this.loadQuote.bind(this));
                this.balanceA = this.root.getElementById('balanceA');

                this.swapDirection = this.root.getElementById('swapDirection');
                this.swapDirection.addEventListener("click", this.changeDirection.bind(this));

                this.tokenBName = this.root.getElementById('bToken');
                this.tokenSwapB = this.root.getElementById('tokenSwapB');
                this.balanceB = this.root.getElementById('balanceB');

                let perButtons = this.root.querySelectorAll('button[data-per]');
                perButtons.forEach(btn => {
                    btn.addEventListener('click', this.inputPercent.bind(this, btn));
                });

                this.quoteWait = this.root.getElementById('quoteWait');

                this.swapBtn = this.root.getElementById('swapBtn');
                this.swapBtn.innerText = currentAccount ? "Swap" : "Connect";
                this.swapBtn.addEventListener('click', this.swap.bind(this));
                this.swapError = this.root.getElementById('swapError');
                
                window.addEventListener('onConnected', event => this.connected(event.detail));
                window.addEventListener('onDisconnected', this.disconnected.bind(this));

                super.loaded = true;
                console.log("Swap loaded");
        }
    }

    async quote(){
        if(!currentAccount){
            //this.swapBtn.disabled = "disabled";
            this.swapBtn.innerText = "Connect";
            this.quoteWait.style.display = 'none';
            return;
        }
    
        this.swapBtn.disabled = "";
        this.swapError.innerText = "";
        this.swapError.style.display = "none";
    
        let amountA = asUnit(this.tokenSwapA.value, 'ether');
    
        let quote = await v2routerContract.amountsOut(amountA, [this.tokenA.address, this.tokenB.address]);
    
        let slip = 1 - ((this.slippage.value ?? 0) / 100);
        this.tokenSwapB.value = (fromWei(quote[1]) * slip).toFixed(18);
    
        let chain = await getChain();
        if(!tokenConfig.test[chain.name][this.tokenA.name()].native) {
            let approved = await this.tokenA.allowance(currentAccount, v2routerContract.address);
            this.swapBtn.innerText = approved.lt(amountA) ? "Approve" : "Swap";
        } else {
            this.swapBtn.innerText = "Swap";
        }
    
        this.quoteWait.style.display = 'none';
    }
    
    async swap(){
        if(!currentAccount){
            await connect();
            return;
        }

        let amountA = asUnit(this.tokenSwapA.value, 'ether');
        let amountB = asUnit(this.tokenSwapB.value, 'ether');

        let btnText = this.swapBtn.innerText;

        try
        {
            let chain = await getChain();
            if(tokenConfig.test[chain.name][this.tokenA.name()].native){
                this.swapBtn.disabled = "disabled";
                this.swapBtn.innerText = "Swapping...";
    
                let tx = await v2routerContract.SwapEthForToken(
                    currentAccount, 
                    amountA, 
                    amountB, 
                    [this.tokenA.address, this.tokenB.address]
                );
                await tx.wait();
    
                this.swapBtn.disabled = "";
                this.swapBtn.innerText = "Swap";
                this.swapError.style.display = 'none';
            }
            else {
                let approved = await this.tokenA.allowance(currentAccount, v2routerContract.address);
    
                if(approved.lt(amountA)){
                    this.swapBtn.disabled = "disabled";
                    this.swapBtn.innerText = "Approving...";
    
                    let tx = await this.tokenA.approve(v2routerContract.address, amountA);
                    await tx.wait();
    
                    this.swapBtn.disabled = "";
                    this.swapBtn.innerText = "Swap";
                    this.swapError.style.display = 'none';
                } else {
                    this.swapBtn.disabled = "disabled";
                    this.swapBtn.innerText = "Swapping...";
    
                    let tx = await v2routerContract.SwapTokenForEthwFee(
                        currentAccount, 
                        amountA, 
                        amountB, 
                        [this.tokenA.address, this.tokenB.address]
                    );
                    await tx.wait();
    
                    this.swapBtn.disabled = "";
                    this.swapBtn.innerText = "Swap";
                    this.swapError.style.display = 'none';
                }    
            }
        } catch (err) {
            this.swapError.innerText = err?.data?.message ?? err?.message;
            this.swapError.style.display = 'unset';
    
            this.swapBtn.innerText = btnText;
            this.swapBtn.disabled = "";
        }
    }
    
    changeDirection(){
        console.log("changing");

        let atkn = this.tokenA;
        this.tokenA = this.tokenB;
        this.tokenB = atkn;

        let aName = this.tokenAName.innerText;
        let aBalance = this.balanceA.innerText;
        let aCache = this.cachebalanceA;

        this.tokenAName.innerText = this.tokenBName.innerText;
        this.balanceA.innerText = this.balanceB.innerText;
        this.cachebalanceA = this.cachebalanceB;

        this.tokenBName.innerText = aName;
        this.balanceB.innerText = aBalance;
        this.cachebalanceB = aCache;
    }
    
    typewatch = function(){
        var timer = 0;
        return function(callback, ms){
            clearTimeout (timer);
            timer = setTimeout(callback, ms);
        }  
    }();
    
    loadQuote(){
        this.quoteWait.style.display = 'unset';
        this.typewatch(this.quote.bind(this), 500);
    }

    async connected(account){
        console.log(`swap detected connection to ${account}`);
        initializeContracts();
        await this.loadPairs();
        this.swapBtn.innerText = "Swap";
    }

    disconnected(){
        console.log("swap detected disconnection");
        this.swapBtn.innerText = "Connect";
    }

    async inputPercent(sender){
        let percentage = sender.getAttribute('data-per');
        if(sender.getAttribute('data-type') == 'a'){
            this.tokenSwapA.value = fromWei(this.cachebalanceA) * (percentage / 100);
        }
        await this.quote();
    }

    async loadBalances(){
        if(currentAccount)
        {
            let chain = await getChain();
            this.cachebalanceA = (tokenConfig.test[chain.name][this.tokenA.name()].native ? await nativeBalance() : await this.tokenA.balanceOf(currentAccount)) ?? 0;
            this.balanceA.innerText = `Balance: ${fromWei(this.cachebalanceA)}`;
            this.cachebalanceB = (tokenConfig.test[chain.name][this.tokenB.name()].native ? await nativeBalance() : await this.tokenB.balanceOf(currentAccount)) ?? 0;
            this.balanceB.innerText = `Balance: ${fromWei(this.cachebalanceB)}`;
        }
    }

    async loadPairs(){
        if(currentAccount && v2routerContract) {
            let chain = await getChain();
            let tokenEntries = Object.entries(tokenConfig.test[chain.name]);
            if(tokenEntries.length >= 2){
                await this.loadA(tokenEntries[0]);
                await this.loadB(tokenEntries[1]);
            }
        }
    }

    async loadA(tokenEntry){
        let [name, tokenSettings] = tokenEntry;
        this.tokenA = new ERC20(tokenSettings.address);
        this.tokenAName.innerText = await this.tokenA.symbol();
        this.cachebalanceA = (tokenSettings.native ? await nativeBalance() : await this.tokenA.balanceOf(currentAccount)) ?? 0;
        this.balanceA.innerText = `Balance: ${fromWei(this.cachebalanceA)}`;
    }

    async loadB(tokenEntry){
        let [name, tokenSettings] = tokenEntry;
        this.tokenB = new ERC20(tokenSettings.address);
        this.tokenBName.innerText = await this.tokenB.symbol();
        this.cachebalanceB = (tokenSettings.native ? await nativeBalance() : await this.tokenB.balanceOf(currentAccount)) ?? 0;
        this.balanceB.innerText = `Balance: ${fromWei(this.cachebalanceB)}`;
    }
}

customElements.define('ohl-swap', Swap);