import { chains } from "@ohlabs/configuration/chain.js";

var currentAccount = null;
var currentChain = null;

var provider = null;
var signer = null;

const onDisconnected = new Event("onDisconnected");

function initializeWallet() {
   if(window.ethereum){
      provider = new ethers.providers.Web3Provider(window.ethereum);
		signer = provider.getSigner();

      // account changed event
      //(accounts: Array<string>) => void
      window.ethereum.on('accountsChanged', accountsChanged);
   
      // chain changed event 
      //(chainId: string) => void
      window.ethereum.on('chainChanged', chainChanged);
   }
}

function accountsChanged(accounts) {
   if (accounts.length === 0) {
      currentAccount =  null;
      window.dispatchEvent(onDisconnected);
   } 
   else {  
      if (accounts[0] !== currentAccount) { currentAccount = accounts[0]; }

      let onConnected = new CustomEvent("onConnected", { detail: currentAccount });
      window.dispatchEvent(onConnected);
   }
}

function chainChanged(chainId) {
   console.log(`chain changed to ${chainId}`);
   currentChain = chainId;
   let onChainChanged = new CustomEvent("onChainChanged", { detail: currentChain });
   window.dispatchEvent(onChainChanged);
}

async function connect() {
   if (window.ethereum) {
      let accounts = await ethereum
                              .request({ method: 'eth_requestAccounts' })
                              .catch((err) => 
                              {
                                 if (err.code === 4001) {
                                    // EIP-1193 userRejectedRequest error
                                    // If this happens, the user rejected the connection request.
                                    console.log('User rejected connection');
                                 } 
                                 else { 
                                    console.error(err); 
                                 }
                              });
      
      accountsChanged(accounts);
      //getChain();    
   } 
   else { 
      console.log("no wallet present"); 
   }
}

async function getAccounts() {
   return await window.ethereum
               .request({ method: 'eth_accounts' })
               .catch((err) => { console.error(err); });
}

async function refreshAccount(){
   if(!currentAccount){
      let accounts = await getAccounts();
      if(accounts){
        currentAccount = accounts[0];
      }
   }
}

async function getChainId() {
   return await window.ethereum
               .request({ method: 'eth_chainId' })
               .catch((err) => { console.error(err); });
}

async function getChain() {
   let chainId = ethers.BigNumber.from(await getChainId()).toNumber();
   let chainName = Object.keys(chains.test)
                        .find(key => chains.test[key] === chainId);
   return { id : chainId, name : chainName };
}

async function hash(message){
   ethers.utils.keccak256(ethers.utils.toUtf8Bytes(message))
}

async function currentBlock(){
   return await provider.getBlockNumber();
}

async function nativeBalance(){
   return await provider.getBalance(currentAccount);
}

async function addToken(contractAddress, contractSymbol, contractDecimals, contractImage){
   try {
      // wasAdded is a boolean. Like any RPC method, an error can be thrown.
      let wasAdded = await ethereum.request({
         method: 'wallet_watchAsset',
         params: {
            type: 'ERC20',
            options: {
            address: contractAddress, // The address of the token.
            symbol: contractSymbol, // A ticker symbol or shorthand, up to 5 characters.
            decimals: contractDecimals, // The number of decimals in the token.
            image: contractImage, // A string URL of the token logo.
            },
         },
      });
   
      if (wasAdded) {
         console.log('Thanks for your interest!');
      } 
   } catch (error) {
      console.log(error);
   }
}

async function addChain(id, name, currency, rpc, explorer){
   try {
      // wasAdded is a boolean. Like any RPC method, an error can be thrown.
      let wasAdded = await ethereum.request({ 
         method: 'wallet_addEthereumChain',
         params:[
            {
               chainId: id,
               chainName: name,
               nativeCurrency: {
                  name: currency.name,
                  symbol: currency.symbol,
                  decimals: currency.decimals //In number form
               },
               rpcUrls: [rpc],
               blockExplorerUrls: [explorer]
            }
         ],
      });
   } catch (error) {
      console.log(error);
   }
}

export { 
   currentAccount, currentChain,
   provider, signer,
   initializeWallet,
   connect, 
   getAccounts, getChain, refreshAccount,
   addToken, addChain,
   hash,
   currentBlock, nativeBalance
}

window.currentBlock = currentBlock;
window.onload = initializeWallet;

window.connect = connect;